@@ -230,6 +230,7 @@ gtk_rotated_bin_realize (GtkWidget *widget)
 {
   GtkRotatedBin *bin = GTK_ROTATED_BIN (widget);
   GtkAllocation allocation;
+  GtkStyleContext *context;
   GdkWindow *window;
   GdkWindowAttr attributes;
   gint attributes_mask;
@@ -278,7 +279,7 @@ gtk_rotated_bin_realize (GtkWidget *widget)
       attributes.width = child_allocation.width;
       attributes.height = child_allocation.height;
     }
-  bin->offscreen_window = gdk_window_new (gdk_screen_get_root_window (gtk_widget_get_screen (widget)),
+  bin->offscreen_window = gdk_window_new (gtk_widget_get_root_window (widget),
                                           &attributes, attributes_mask);
   gdk_window_set_user_data (bin->offscreen_window, widget);
   if (bin->child)
@@ -289,6 +290,9 @@ gtk_rotated_bin_realize (GtkWidget *widget)
   g_signal_connect (bin->offscreen_window, "from-embedder",
                     G_CALLBACK (offscreen_window_from_parent), bin);
 
+  context = gtk_widget_get_style_context (widget);
+  gtk_style_context_set_background (context, window);
+  gtk_style_context_set_background (context, bin->offscreen_window);
   gdk_window_show (bin->offscreen_window);
 }
 
@@ -328,7 +332,7 @@ gtk_rotated_bin_add (GtkContainer *container,
       bin->child = widget;
     }
   else
-    g_warning ("GtkRotatedBin cannot have more than one child");
+    g_warning ("GtkRotatedBin cannot have more than one child\n");
 }
 
 static void
@@ -561,23 +565,26 @@ scale_changed (GtkRange      *range,
   gtk_rotated_bin_set_angle (bin, gtk_range_get_value (range));
 }
 
+static GtkWidget *window = NULL;
+
 GtkWidget *
 do_offscreen_window (GtkWidget *do_widget)
 {
-  static GtkWidget *window = NULL;
-
   if (!window)
     {
       GtkWidget *bin, *vbox, *scale, *button;
+      GdkRGBA black;
 
       window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
       gtk_window_set_screen (GTK_WINDOW (window),
                              gtk_widget_get_screen (do_widget));
-      gtk_window_set_title (GTK_WINDOW (window), "Rotated Button");
+      gtk_window_set_title (GTK_WINDOW (window), "Rotated widget");
 
       g_signal_connect (window, "destroy",
                         G_CALLBACK (gtk_widget_destroyed), &window);
 
+      gdk_rgba_parse (&black, "black");
+      gtk_widget_override_background_color (window, 0, &black);
       gtk_container_set_border_width (GTK_CONTAINER (window), 10);
 
       vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
@@ -599,7 +606,10 @@ do_offscreen_window (GtkWidget *do_widget)
   if (!gtk_widget_get_visible (window))
     gtk_widget_show_all (window);
   else
-    gtk_widget_destroy (window);
+    {
+      gtk_widget_destroy (window);
+      window = NULL;
+    }
 
   return window;
 }
