@@ -12,9 +12,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -69,8 +67,6 @@
  * they will stay exactly synchronized.
  */
 
-G_DEFINE_ABSTRACT_TYPE (GdkFrameClock, gdk_frame_clock, G_TYPE_OBJECT)
-
 enum {
   FLUSH_EVENTS,
   BEFORE_PAINT,
@@ -94,6 +90,8 @@ struct _GdkFrameClockPrivate
   GdkFrameTimings *timings[FRAME_HISTORY_MAX_LENGTH];
 };
 
+G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GdkFrameClock, gdk_frame_clock, G_TYPE_OBJECT)
+
 static void
 gdk_frame_clock_finalize (GObject *object)
 {
@@ -236,8 +234,6 @@ gdk_frame_clock_class_init (GdkFrameClockClass *klass)
                   NULL, NULL,
                   g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
-
-  g_type_class_add_private (klass, sizeof (GdkFrameClockPrivate));
 }
 
 static void
@@ -245,10 +241,7 @@ gdk_frame_clock_init (GdkFrameClock *clock)
 {
   GdkFrameClockPrivate *priv;
 
-  clock->priv = G_TYPE_INSTANCE_GET_PRIVATE (clock,
-                                             GDK_TYPE_FRAME_CLOCK,
-                                             GdkFrameClockPrivate);
-  priv = clock->priv;
+  clock->priv = priv = gdk_frame_clock_get_instance_private (clock);
 
   priv->frame_counter = -1;
   priv->current = FRAME_HISTORY_MAX_LENGTH - 1;
