@@ -1048,12 +1048,12 @@ GDK_AVAILABLE_IN_ALL
 void       gdk_window_set_debug_updates   (gboolean      setting);
 
 GDK_AVAILABLE_IN_ALL
-void       gdk_window_constrain_size      (GdkGeometry  *geometry,
-                                           guint         flags,
-                                           gint          width,
-                                           gint          height,
-                                           gint         *new_width,
-                                           gint         *new_height);
+void       gdk_window_constrain_size      (GdkGeometry    *geometry,
+                                           GdkWindowHints  flags,
+                                           gint            width,
+                                           gint            height,
+                                           gint           *new_width,
+                                           gint           *new_height);
 
 GDK_DEPRECATED_IN_3_8
 void gdk_window_enable_synchronized_configure (GdkWindow *window);
@@ -1096,6 +1096,13 @@ void       gdk_window_set_event_compression    (GdkWindow      *window,
 GDK_AVAILABLE_IN_3_12
 gboolean   gdk_window_get_event_compression    (GdkWindow      *window);
 
+GDK_AVAILABLE_IN_3_12
+void       gdk_window_set_shadow_width         (GdkWindow      *window,
+                                                gint            left,
+                                                gint            right,
+                                                gint            top,
+                                                gint            bottom);
+
 G_END_DECLS
 
 #endif /* __GDK_WINDOW_H__ */
