@@ -65,7 +65,7 @@ void     gdk_x11_window_set_utf8_property    (GdkWindow *window,
 GDK_AVAILABLE_IN_3_2
 void     gdk_x11_window_set_theme_variant (GdkWindow   *window,
                                            char        *variant);
-GDK_AVAILABLE_IN_3_10
+GDK_DEPRECATED_IN_3_12_FOR(gdk_window_set_shadow_width)
 void     gdk_x11_window_set_frame_extents (GdkWindow *window,
                                            int        left,
                                            int        right,
@@ -77,6 +77,12 @@ void     gdk_x11_window_set_hide_titlebar_when_maximized (GdkWindow *window,
 GDK_AVAILABLE_IN_ALL
 void     gdk_x11_window_move_to_current_desktop (GdkWindow   *window);
 
+GDK_AVAILABLE_IN_3_10
+guint32  gdk_x11_window_get_desktop             (GdkWindow   *window);
+GDK_AVAILABLE_IN_3_10
+void     gdk_x11_window_move_to_desktop         (GdkWindow   *window,
+                                                 guint32      desktop);
+
 GDK_AVAILABLE_IN_3_8
 void     gdk_x11_window_set_frame_sync_enabled (GdkWindow *window,
                                                 gboolean   frame_sync_enabled);
