@@ -66,7 +66,7 @@
  * opened when the user selects the <literal>About</literal> option from
  * the <literal>Help</literal> menu. All parts of the dialog are optional.
  *
- * About dialog often contain links and email addresses. GtkAboutDialog
+ * About dialogs often contain links and email addresses. GtkAboutDialog
  * displays these as clickable links. By default, it calls gtk_show_uri()
  * when a user clicks one. The behaviour can be overridden with the
  * #GtkAboutDialog::activate-link signal.
@@ -94,25 +94,31 @@
  */
 
 /* Translators: this is the license preamble; the string at the end
- * contains the URL of the license.
+ * contains the name of the license as link text.
  */
-static const gchar *gtk_license_preamble = N_("This program comes with ABSOLUTELY NO WARRANTY;\nfor details, visit <a href=\"%s\">%s</a>");
+static const gchar *gtk_license_preamble = N_("This program comes with ABSOLUTELY NO WARRANTY.\nSee the <a href=\"%s\">%s</a> for details.");
 
-/* URLs for each GtkLicense type; keep in the same order as the enumeration */
-static const gchar *gtk_license_urls[] = {
-  NULL,
-  NULL,
+typedef struct
+{
+  const gchar *name;
+  const gchar *url;
+} LicenseInfo;
 
-  "http://www.gnu.org/licenses/old-licenses/gpl-2.0.html",
-  "http://www.gnu.org/licenses/gpl.html",
-
-  "http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html",
-  "http://www.gnu.org/licenses/lgpl.html",
-
-  "http://opensource.org/licenses/bsd-license.php",
-  "http://opensource.org/licenses/mit-license.php",
-
-  "http://opensource.org/licenses/artistic-license-2.0.php"
+/* LicenseInfo for each GtkLicense type; keep in the same order as the enumeration */
+static const LicenseInfo gtk_license_info [] = {
+  { N_("License"), NULL },
+  { N_("Custom License") , NULL },
+  { N_("GNU General Public License, version 2 or later"), "http://www.gnu.org/licenses/old-licenses/gpl-2.0.html" },
+  { N_("GNU General Public License, version 3 or later"), "http://www.gnu.org/licenses/gpl.html" },
+  { N_("GNU Lesser General Public License, version 2.1 or later"), "http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html" },
+  { N_("GNU Lesser General Public License, version 3 or later"), "http://www.gnu.org/licenses/lgpl.html" },
+  { N_("BSD 2-Clause License"), "http://opensource.org/licenses/bsd-license.php" },
+  { N_("The MIT License (MIT)"), "http://opensource.org/licenses/mit-license.php" },
+  { N_("Artistic License 2.0"), "http://opensource.org/licenses/artistic-license-2.0.php" },
+  { N_("GNU General Public License, version 2 only"), "http://www.gnu.org/licenses/old-licenses/gpl-2.0.html" },
+  { N_("GNU General Public License, version 3 only"), "http://www.gnu.org/licenses/gpl.html" },
+  { N_("GNU Lesser General Public License, version 2.1 only"), "http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html" },
+  { N_("GNU Lesser General Public License, version 3 only"), "http://www.gnu.org/licenses/lgpl.html" }
 };
 
 typedef struct
@@ -1778,8 +1784,7 @@ gtk_about_dialog_set_logo_icon_name (GtkAboutDialog *about,
                                      const gchar    *icon_name)
 {
   GtkAboutDialogPrivate *priv;
-  gint *sizes;
-  gint i, best_size;
+  GList *icons;
 
   g_return_if_fail (GTK_IS_ABOUT_DIALOG (about));
 
@@ -1790,33 +1795,48 @@ gtk_about_dialog_set_logo_icon_name (GtkAboutDialog *about,
   if (gtk_image_get_storage_type (GTK_IMAGE (priv->logo_image)) == GTK_IMAGE_PIXBUF)
     g_object_notify (G_OBJECT (about), "logo");
 
-  sizes = gtk_icon_theme_get_icon_sizes (gtk_icon_theme_get_default (), icon_name);
-  best_size = 0;
-  for (i = 0; sizes[i]; i++)
+  if (icon_name)
     {
-      if (sizes[i] >= 128 || sizes[i] == -1)
-        {
-          best_size = 128;
-          break;
-        }
-      else if (sizes[i] >= 96)
-        {
-          best_size = MAX (96, best_size);
-        }
-      else if (sizes[i] >= 64)
-        {
-          best_size = MAX (64, best_size);
-        }
-      else
-        {
-          best_size = MAX (48, best_size);
-        }
-    }
-  g_free (sizes);
+      gint *sizes = gtk_icon_theme_get_icon_sizes (gtk_icon_theme_get_default (),
+                                                   icon_name);
+      gint i, best_size = 0;
 
-  gtk_image_set_from_icon_name (GTK_IMAGE (priv->logo_image), icon_name,
+      for (i = 0; sizes[i]; i++)
+        {
+          if (sizes[i] >= 128 || sizes[i] == -1)
+            {
+              best_size = 128;
+              break;
+            }
+          else if (sizes[i] >= 96)
+            {
+              best_size = MAX (96, best_size);
+            }
+          else if (sizes[i] >= 64)
+            {
+              best_size = MAX (64, best_size);
+            }
+          else
+            {
+              best_size = MAX (48, best_size);
+            }
+        }
+      g_free (sizes);
+
+      gtk_image_set_from_icon_name (GTK_IMAGE (priv->logo_image), icon_name,
                                 GTK_ICON_SIZE_DIALOG);
-  gtk_image_set_pixel_size (GTK_IMAGE (priv->logo_image), best_size);
+      gtk_image_set_pixel_size (GTK_IMAGE (priv->logo_image), best_size);
+    }
+  else if ((icons = gtk_window_get_default_icon_list ()))
+    {
+      gtk_image_set_from_pixbuf (GTK_IMAGE (priv->logo_image), icons->data);
+      g_list_free (icons);
+    }
+  else
+    {
+      gtk_image_clear (GTK_IMAGE (priv->logo_image));
+    }
+
   g_object_notify (G_OBJECT (about), "logo-icon-name");
 
   g_object_thaw_notify (G_OBJECT (about));
@@ -2241,6 +2261,8 @@ add_credits_section (GtkAboutDialog *about,
 
       label = gtk_label_new (str->str);
       gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+      g_signal_connect_swapped (label, "activate-link",
+                                G_CALLBACK (emit_activate_link), about);
       g_string_free (str, TRUE);
       gtk_widget_set_halign (label, GTK_ALIGN_START);
       gtk_widget_set_valign (label, GTK_ALIGN_CENTER);
@@ -2465,16 +2487,18 @@ gtk_about_dialog_set_license_type (GtkAboutDialog *about,
       /* custom licenses use the contents of the :license property */
       if (priv->license_type != GTK_LICENSE_CUSTOM)
         {
+          const gchar *name;
           const gchar *url;
           gchar *license_string;
           GString *str;
 
-          url = gtk_license_urls[priv->license_type];
+          name = _(gtk_license_info[priv->license_type].name);
+          url = gtk_license_info[priv->license_type].url;
           if (url == NULL)
             url = priv->website_url;
 
           str = g_string_sized_new (256);
-          g_string_append_printf (str, _(gtk_license_preamble), url, url);
+          g_string_append_printf (str, _(gtk_license_preamble), url, name);
 
           g_free (priv->license);
           priv->license = g_string_free (str, FALSE);
