@@ -43,8 +43,6 @@
 
 #include "gtkintl.h"
 #include "gtkappchooser.h"
-#include "gtkappchooseronline.h"
-#include "gtkappchooserprivate.h"
 #include "gtkappchooserprivate.h"
 
 #include "gtkmessagedialog.h"
@@ -52,7 +50,6 @@
 #include "gtkbbox.h"
 #include "gtkbutton.h"
 #include "gtkmenuitem.h"
-#include "gtkstock.h"
 
 #include <string.h>
 #include <glib/gi18n-lib.h>
@@ -67,7 +64,7 @@ struct _GtkAppChooserDialogPrivate {
 
   GtkWidget *label;
   GtkWidget *button;
-  GtkWidget *online_button;
+  GtkWidget *software_button;
   GtkWidget *inner_box;
 
   GtkWidget *open_label;
@@ -75,9 +72,6 @@ struct _GtkAppChooserDialogPrivate {
   GtkWidget *app_chooser_widget;
   GtkWidget *show_more_button;
 
-  GtkAppChooserOnline *online;
-  GCancellable *online_cancellable;
-
   gboolean show_more_clicked;
   gboolean dismissed;
 };
@@ -94,125 +88,6 @@ G_DEFINE_TYPE_WITH_CODE (GtkAppChooserDialog, gtk_app_chooser_dialog, GTK_TYPE_D
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_APP_CHOOSER,
                                                 gtk_app_chooser_dialog_iface_init));
 
-static void
-show_error_dialog (const gchar *primary,
-                   const gchar *secondary,
-                   GtkWindow   *parent)
-{
-  GtkWidget *message_dialog;
-
-  message_dialog = gtk_message_dialog_new (parent, 0,
-                                           GTK_MESSAGE_ERROR,
-                                           GTK_BUTTONS_OK,
-                                           NULL);
-  g_object_set (message_dialog,
-                "text", primary,
-                "secondary-text", secondary,
-                NULL);
-  gtk_dialog_set_default_response (GTK_DIALOG (message_dialog), GTK_RESPONSE_OK);
-
-  gtk_widget_show (message_dialog);
-
-  g_signal_connect (message_dialog, "response",
-                    G_CALLBACK (gtk_widget_destroy), NULL);
-}
-
-static void
-search_for_mimetype_ready_cb (GObject      *source,
-                              GAsyncResult *res,
-                              gpointer      user_data)
-{
-  GtkAppChooserOnline *online = GTK_APP_CHOOSER_ONLINE (source);
-  GtkAppChooserDialog *self = user_data;
-  GError *error = NULL;
-
-  gdk_threads_enter ();
-
-  _gtk_app_chooser_online_search_for_mimetype_finish (online, res, &error);
-
-  if (self->priv->dismissed)
-    goto out;
-
-  if (error != NULL &&
-      !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
-    {
-      show_error_dialog (_("Failed to look for applications online"),
-                         error->message, GTK_WINDOW (self));
-    }
-  else
-    {
-      gtk_widget_set_sensitive (self->priv->online_button, TRUE);
-      gtk_app_chooser_refresh (GTK_APP_CHOOSER (self->priv->app_chooser_widget));
-    }
-
- out:
-  g_clear_object (&self->priv->online_cancellable);
-  g_clear_error (&error);
-  g_object_unref (self);
-
-  gdk_threads_leave ();
-}
-
-static void
-online_button_clicked_cb (GtkButton *b,
-                          gpointer   user_data)
-{
-  GtkAppChooserDialog *self = user_data;
-
-  self->priv->online_cancellable = g_cancellable_new ();
-  gtk_widget_set_sensitive (self->priv->online_button, FALSE);
-
-  _gtk_app_chooser_online_search_for_mimetype_async (self->priv->online,
-						     self->priv->content_type,
-						     GTK_WINDOW (self),
-                                                     self->priv->online_cancellable,
-						     search_for_mimetype_ready_cb,
-						     g_object_ref (self));
-}
-
-static void
-app_chooser_online_get_default_ready_cb (GObject *source,
-                                         GAsyncResult *res,
-                                         gpointer user_data)
-{
-  GtkAppChooserDialog *self = user_data;
-
-  gdk_threads_enter ();
-
-  self->priv->online = _gtk_app_chooser_online_get_default_finish (source, res);
-
-  if (self->priv->online != NULL &&
-      !self->priv->dismissed)
-    {
-      GtkWidget *action_area;
-
-      action_area = gtk_dialog_get_action_area (GTK_DIALOG (self));
-      self->priv->online_button = gtk_button_new_with_mnemonic (_("_Find applications online"));
-      gtk_box_pack_start (GTK_BOX (action_area), self->priv->online_button,
-                          FALSE, FALSE, 0);
-      gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (action_area), self->priv->online_button,
-                                          TRUE);
-      g_signal_connect (self->priv->online_button, "clicked",
-                        G_CALLBACK (online_button_clicked_cb), self);
-
-
-      if (!self->priv->content_type)
-	gtk_widget_set_sensitive (self->priv->online_button, FALSE);
-
-      gtk_widget_show (self->priv->online_button);
-    }
-
-  g_object_unref (self);
-
-  gdk_threads_leave ();
-}
-
-static void
-ensure_online_button (GtkAppChooserDialog *self)
-{
-  _gtk_app_chooser_online_get_default_async (app_chooser_online_get_default_ready_cb,
-                                             g_object_ref (self));
-}
 
 static void
 add_or_find_application (GtkAppChooserDialog *self)
@@ -232,16 +107,6 @@ add_or_find_application (GtkAppChooserDialog *self)
     }
 }
 
-static void
-cancel_and_clear_cancellable (GtkAppChooserDialog *self)
-{                                                               
-  if (self->priv->online_cancellable != NULL)
-    {
-      g_cancellable_cancel (self->priv->online_cancellable);
-      g_clear_object (&self->priv->online_cancellable);
-    }
-}
-
 static void
 gtk_app_chooser_dialog_response (GtkDialog *dialog,
                                  gint       response_id,
@@ -256,7 +121,6 @@ gtk_app_chooser_dialog_response (GtkDialog *dialog,
       break;
     case GTK_RESPONSE_CANCEL:
     case GTK_RESPONSE_DELETE_EVENT:
-      cancel_and_clear_cancellable (self);
       self->priv->dismissed = TRUE;
     default :
       break;
@@ -331,16 +195,16 @@ set_dialog_properties (GtkAppChooserDialog *self)
   if (name != NULL)
     {
       /* Translators: %s is a filename */
-      label = g_strdup_printf (_("Select an application to open \"%s\""), name);
-      string = g_strdup_printf (_("No applications available to open \"%s\""),
+      label = g_strdup_printf (_("Select an application to open “%s”"), name);
+      string = g_strdup_printf (_("No applications available to open “%s”"),
                                 name);
     }
   else
     {
       /* Translators: %s is a file type description */
-      label = g_strdup_printf (_("Select an application for \"%s\" files"),
+      label = g_strdup_printf (_("Select an application for “%s” files"),
                                unknown ? self->priv->content_type : description);
-      string = g_strdup_printf (_("No applications available to open \"%s\" files"),
+      string = g_strdup_printf (_("No applications available to open “%s” files"),
                                unknown ? self->priv->content_type : description);
     }
 
@@ -357,7 +221,7 @@ set_dialog_properties (GtkAppChooserDialog *self)
   default_text = g_strdup_printf ("<big><b>%s</b></big>\n%s",
                                   string,
                                   _("Click \"Show other applications\", for more options, or "
-                                    "\"Find applications online\" to install a new application"));
+                                    "\"Software\" to install a new application"));
 
   gtk_app_chooser_widget_set_default_text (GTK_APP_CHOOSER_WIDGET (self->priv->app_chooser_widget),
                                            default_text);
@@ -483,7 +347,7 @@ construct_appchooser_widget (GtkAppChooserDialog *self)
 
 static void
 set_gfile_and_content_type (GtkAppChooserDialog *self,
-                            GFile *file)
+                            GFile               *file)
 {
   GFileInfo *info;
 
@@ -515,6 +379,78 @@ gtk_app_chooser_dialog_refresh (GtkAppChooser *object)
   gtk_app_chooser_refresh (GTK_APP_CHOOSER (self->priv->app_chooser_widget));
 }
 
+static void
+show_error_dialog (const gchar *primary,
+                   const gchar *secondary,
+                   GtkWindow   *parent)
+{
+  GtkWidget *message_dialog;
+
+  message_dialog = gtk_message_dialog_new (parent, 0,
+                                           GTK_MESSAGE_ERROR,
+                                           GTK_BUTTONS_OK,
+                                           NULL);
+  g_object_set (message_dialog,
+                "text", primary,
+                "secondary-text", secondary,
+                NULL);
+  gtk_dialog_set_default_response (GTK_DIALOG (message_dialog), GTK_RESPONSE_OK);
+
+  gtk_widget_show (message_dialog);
+
+  g_signal_connect (message_dialog, "response",
+                    G_CALLBACK (gtk_widget_destroy), NULL);
+}
+
+static void
+software_button_clicked_cb (GtkButton           *button,
+                            GtkAppChooserDialog *self)
+{
+  GSubprocess *process;
+  GError *error = NULL;
+  gchar *option;
+
+  if (self->priv->content_type)
+    option = g_strconcat ("--search=", self->priv->content_type, NULL);
+  else
+    option = g_strdup ("--mode=overview");
+
+  process = g_subprocess_new (0, &error, "gnome-software", option, NULL);
+  if (!process)
+    {
+      show_error_dialog (_("Failed to start GNOME Software"),
+                         error->message, GTK_WINDOW (self));
+      g_error_free (error);
+    }
+  else
+    g_object_unref (process);
+
+  g_free (option);
+}
+
+static void
+ensure_software_button (GtkAppChooserDialog *self)
+{
+  if (g_find_program_in_path ("gnome-software"))
+    {
+      GtkWidget *action_area;
+
+      action_area = gtk_dialog_get_action_area (GTK_DIALOG (self));
+      self->priv->software_button = gtk_button_new_with_mnemonic (_("Software"));
+      gtk_button_set_always_show_image (GTK_BUTTON (self->priv->software_button), TRUE);
+      gtk_button_set_image (GTK_BUTTON (self->priv->software_button), gtk_image_new_from_icon_name ("gnome-software", GTK_ICON_SIZE_BUTTON));
+
+      gtk_box_pack_start (GTK_BOX (action_area), self->priv->software_button,
+                          FALSE, FALSE, 0);
+      gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (action_area), self->priv->software_button,
+                                          TRUE);
+      g_signal_connect (self->priv->software_button, "clicked",
+                        G_CALLBACK (software_button_clicked_cb), self);
+
+      gtk_widget_show (self->priv->software_button);
+    }
+}
+
 static void
 gtk_app_chooser_dialog_constructed (GObject *object)
 {
@@ -525,7 +461,7 @@ gtk_app_chooser_dialog_constructed (GObject *object)
 
   construct_appchooser_widget (self);
   set_dialog_properties (self);
-  ensure_online_button (self);
+  ensure_software_button (self);
 }
 
 static void
@@ -534,8 +470,6 @@ gtk_app_chooser_dialog_dispose (GObject *object)
   GtkAppChooserDialog *self = GTK_APP_CHOOSER_DIALOG (object);
   
   g_clear_object (&self->priv->gfile);
-  cancel_and_clear_cancellable (self);
-  g_clear_object (&self->priv->online);
 
   self->priv->dismissed = TRUE;
 
@@ -661,11 +595,11 @@ gtk_app_chooser_dialog_class_init (GtkAppChooserDialogClass *klass)
   widget_class = GTK_WIDGET_CLASS (klass);
   gtk_widget_class_set_template_from_resource (widget_class,
 					       "/org/gtk/libgtk/gtkappchooserdialog.ui");
-  gtk_widget_class_bind_child (widget_class, GtkAppChooserDialogPrivate, label);
-  gtk_widget_class_bind_child (widget_class, GtkAppChooserDialogPrivate, button);
-  gtk_widget_class_bind_child (widget_class, GtkAppChooserDialogPrivate, show_more_button);
-  gtk_widget_class_bind_child (widget_class, GtkAppChooserDialogPrivate, inner_box);
-  gtk_widget_class_bind_callback (widget_class, show_more_button_clicked_cb);
+  gtk_widget_class_bind_template_child_private (widget_class, GtkAppChooserDialog, label);
+  gtk_widget_class_bind_template_child_private (widget_class, GtkAppChooserDialog, button);
+  gtk_widget_class_bind_template_child_private (widget_class, GtkAppChooserDialog, show_more_button);
+  gtk_widget_class_bind_template_child_private (widget_class, GtkAppChooserDialog, inner_box);
+  gtk_widget_class_bind_template_callback (widget_class, show_more_button_clicked_cb);
 }
 
 static void
