@@ -21,7 +21,6 @@
 #include "config.h"
 
 #include "gtkapplicationprivate.h"
-#include "gtkbuilder.h"
 #import <Cocoa/Cocoa.h>
 
 typedef struct
@@ -56,18 +55,14 @@ typedef struct
 } GtkApplicationImplQuartz;
 
 G_DEFINE_TYPE (GtkApplicationImplQuartz, gtk_application_impl_quartz, GTK_TYPE_APPLICATION_IMPL)
-#if MAC_OS_X_VERSION_MIN_REQUIRED >= 1060
-@interface GtkApplicationQuartzDelegate : NSObject <NSApplicationDelegate>
-#else
+
 @interface GtkApplicationQuartzDelegate : NSObject
-#endif
 {
   GtkApplicationImplQuartz *quartz;
 }
 
 - (id)initWithImpl:(GtkApplicationImplQuartz*)impl;
 - (NSApplicationTerminateReply) applicationShouldTerminate:(NSApplication *)sender;
-- (void)application:(NSApplication *)theApplication openFiles:(NSArray *)filenames;
 @end
 
 @implementation GtkApplicationQuartzDelegate
@@ -80,135 +75,35 @@ G_DEFINE_TYPE (GtkApplicationImplQuartz, gtk_application_impl_quartz, GTK_TYPE_A
 
 -(NSApplicationTerminateReply) applicationShouldTerminate:(NSApplication *)sender
 {
-  const gchar *quit_action_name = "quit";
-  GActionGroup *action_group = G_ACTION_GROUP (quartz->impl.application);
-
-  if (quartz->quit_inhibit != 0)
-    return NSTerminateCancel;
-
-  if (g_action_group_has_action (action_group, quit_action_name))
-    {
-      g_action_group_activate_action (action_group, quit_action_name, NULL);
-      return NSTerminateCancel;
-    }
-
-  return NSTerminateNow;
-}
-
--(void)application:(NSApplication *)theApplication openFiles:(NSArray *)filenames
-{
-  GFile **files;
-  gint i;
-  GApplicationFlags flags;
-
-  flags = g_application_get_flags (G_APPLICATION (quartz->impl.application));
-
-  if (~flags & G_APPLICATION_HANDLES_OPEN)
-    {
-      [theApplication replyToOpenOrPrint:NSApplicationDelegateReplyFailure];
-      return;
-    }
-
-  files = g_new (GFile *, [filenames count]);
-
-  for (i = 0; i < [filenames count]; i++)
-    files[i] = g_file_new_for_path ([(NSString *)[filenames objectAtIndex:i] UTF8String]);
-
-  g_application_open (G_APPLICATION (quartz->impl.application), files, [filenames count], "");
-
-  for (i = 0; i < [filenames count]; i++)
-    g_object_unref (files[i]);
-
-  g_free (files);
-
-  [theApplication replyToOpenOrPrint:NSApplicationDelegateReplySuccess];
+  /* We have no way to give our message other than to pop up a dialog
+   * ourselves, which we should not do since the OS will already show
+   * one when we return NSTerminateNow.
+   *
+   * Just let the OS show the generic message...
+   */
+  return quartz->quit_inhibit == 0 ? NSTerminateNow : NSTerminateCancel;
 }
 @end
 
-/* these exist only for accel handling */
-static void
-gtk_application_impl_quartz_hide (GSimpleAction *action,
-                                  GVariant      *parameter,
-                                  gpointer       user_data)
-{
-  [NSApp hide:NSApp];
-}
-
-static void
-gtk_application_impl_quartz_hide_others (GSimpleAction *action,
-                                         GVariant      *parameter,
-                                         gpointer       user_data)
-{
-  [NSApp hideOtherApplications:NSApp];
-}
-
-static void
-gtk_application_impl_quartz_show_all (GSimpleAction *action,
-                                      GVariant      *parameter,
-                                      gpointer       user_data)
-{
-  [NSApp unhideAllApplications:NSApp];
-}
-
-static GActionEntry gtk_application_impl_quartz_actions[] = {
-  { "hide",             gtk_application_impl_quartz_hide        },
-  { "hide-others",      gtk_application_impl_quartz_hide_others },
-  { "show-all",         gtk_application_impl_quartz_show_all    }
-};
-
 static void
 gtk_application_impl_quartz_startup (GtkApplicationImpl *impl,
                                      gboolean            register_session)
 {
   GtkApplicationImplQuartz *quartz = (GtkApplicationImplQuartz *) impl;
-  GSimpleActionGroup *gtkinternal;
-  GMenuModel *app_menu;
-  const gchar *pref_accel[] = {"<Primary>comma", NULL};
-  const gchar *hide_others_accel[] = {"<Primary><Alt>h", NULL};
-  const gchar *hide_accel[] = {"<Primary>h", NULL};
-  const gchar *quit_accel[] = {"<Primary>q", NULL};
 
   if (register_session)
     {
       quartz->delegate = [[GtkApplicationQuartzDelegate alloc] initWithImpl:quartz];
-      [NSApp setDelegate: (id)(quartz->delegate)];
+      [NSApp setDelegate: quartz->delegate];
     }
 
   quartz->muxer = gtk_action_muxer_new ();
   gtk_action_muxer_set_parent (quartz->muxer, gtk_application_get_action_muxer (impl->application));
 
-  /* Add the default accels */
-  gtk_application_set_accels_for_action (impl->application, "app.preferences", pref_accel);
-  gtk_application_set_accels_for_action (impl->application, "gtkinternal.hide-others", hide_others_accel);
-  gtk_application_set_accels_for_action (impl->application, "gtkinternal.hide", hide_accel);
-  gtk_application_set_accels_for_action (impl->application, "app.quit", quit_accel);
-
-  /* and put code behind the 'special' accels */
-  gtkinternal = g_simple_action_group_new ();
-  g_action_map_add_action_entries (G_ACTION_MAP (gtkinternal), gtk_application_impl_quartz_actions,
-                                   G_N_ELEMENTS (gtk_application_impl_quartz_actions), quartz);
-  gtk_application_insert_action_group (impl->application, "gtkinternal", G_ACTION_GROUP (gtkinternal));
-  g_object_unref (gtkinternal);
-
-  /* now setup the menu */
-  app_menu = gtk_application_get_app_menu (impl->application);
-  if (app_menu == NULL)
-    {
-      GtkBuilder *builder;
-
-      /* If the user didn't fill in their own menu yet, add ours.
-       *
-       * The fact that we do this here ensures that we will always have the
-       * app menu at index 0 in 'combined'.
-       */
-      builder = gtk_builder_new_from_resource ("/org/gtk/libgtk/ui/gtkapplication-quartz.ui");
-      gtk_application_set_app_menu (impl->application, G_MENU_MODEL (gtk_builder_get_object (builder, "app-menu")));
-      g_object_unref (builder);
-    }
-  else
-    gtk_application_impl_set_app_menu (impl, app_menu);
-
-  /* This may or may not add an item to 'combined' */
+  /* app menu must come first so that we always see index '0' in
+   * 'combined' as being the app menu.
+   */
+  gtk_application_impl_set_app_menu (impl, gtk_application_get_app_menu (impl->application));
   gtk_application_impl_set_menubar (impl, gtk_application_get_menubar (impl->application));
 
   /* OK.  Now put it in the menu. */
@@ -346,13 +241,6 @@ gtk_application_impl_quartz_is_inhibited (GtkApplicationImpl         *impl,
 static void
 gtk_application_impl_quartz_init (GtkApplicationImplQuartz *quartz)
 {
-  /* This is required so that Cocoa is not going to parse the
-     command line arguments by itself and generate OpenFile events.
-     We already parse the command line ourselves, so this is needed
-     to prevent opening files twice, etc. */
-  [[NSUserDefaults standardUserDefaults] setObject:@"NO"
-                                            forKey:@"NSTreatUnknownArgumentsAsOpen"];
-
   quartz->combined = g_menu_new ();
 }
 
