@@ -29,14 +29,6 @@
 #include "gtkintl.h"
 #include "gtksettings.h"
 
-#include <gdk/gdk.h>
-#ifdef GDK_WINDOWING_X11
-#include <gdk/x11/gdkx.h>
-#endif
-#ifdef GDK_WINDOWING_WAYLAND
-#include <gdk/wayland/gdkwayland.h>
-#endif
-
 #ifdef HAVE_GIO_UNIX
 #include <gio/gdesktopappinfo.h>
 #endif
@@ -84,7 +76,7 @@
  * property. If the desktop environment does not display the application
  * menu, then it will automatically be included in the menubar. It can
  * also be shown as part of client-side window decorations, e.g. by
- * using gtk_header_bar_set_show_fallback_app_menu().
+ * using gtk_header_bar_set_show_close_button().
  *
  * <example><title>A GtkApplicationWindow with a menubar</title>
  * <programlisting><![CDATA[
@@ -223,10 +215,6 @@ struct _GtkApplicationWindowPrivate
   GMenu *app_menu_section;
   GMenu *menubar_section;
 
-  GDBusConnection *session;
-  gchar           *object_path;
-  guint            export_id;
-
   guint            id;
 };
 
@@ -401,6 +389,10 @@ gtk_application_window_list_actions (GActionGroup *group)
 {
   GtkApplicationWindow *window = GTK_APPLICATION_WINDOW (group);
 
+  /* may be NULL after dispose has run */
+  if (!window->priv->actions)
+    return g_new0 (char *, 0 + 1);
+
   return g_action_group_list_actions (G_ACTION_GROUP (window->priv->actions));
 }
 
@@ -415,6 +407,9 @@ gtk_application_window_query_action (GActionGroup        *group,
 {
   GtkApplicationWindow *window = GTK_APPLICATION_WINDOW (group);
 
+  if (!window->priv->actions)
+    return FALSE;
+
   return g_action_group_query_action (G_ACTION_GROUP (window->priv->actions),
                                       action_name, enabled, parameter_type, state_type, state_hint, state);
 }
@@ -426,7 +421,10 @@ gtk_application_window_activate_action (GActionGroup *group,
 {
   GtkApplicationWindow *window = GTK_APPLICATION_WINDOW (group);
 
-  return g_action_group_activate_action (G_ACTION_GROUP (window->priv->actions), action_name, parameter);
+  if (!window->priv->actions)
+    return;
+
+  g_action_group_activate_action (G_ACTION_GROUP (window->priv->actions), action_name, parameter);
 }
 
 static void
@@ -436,7 +434,10 @@ gtk_application_window_change_action_state (GActionGroup *group,
 {
   GtkApplicationWindow *window = GTK_APPLICATION_WINDOW (group);
 
-  return g_action_group_change_action_state (G_ACTION_GROUP (window->priv->actions), action_name, state);
+  if (!window->priv->actions)
+    return;
+
+  g_action_group_change_action_state (G_ACTION_GROUP (window->priv->actions), action_name, state);
 }
 
 static GAction *
@@ -445,6 +446,9 @@ gtk_application_window_lookup_action (GActionMap  *action_map,
 {
   GtkApplicationWindow *window = GTK_APPLICATION_WINDOW (action_map);
 
+  if (!window->priv->actions)
+    return NULL;
+
   return g_action_map_lookup_action (G_ACTION_MAP (window->priv->actions), action_name);
 }
 
@@ -454,6 +458,9 @@ gtk_application_window_add_action (GActionMap *action_map,
 {
   GtkApplicationWindow *window = GTK_APPLICATION_WINDOW (action_map);
 
+  if (!window->priv->actions)
+    return;
+
   g_action_map_add_action (G_ACTION_MAP (window->priv->actions), action);
 }
 
@@ -463,6 +470,9 @@ gtk_application_window_remove_action (GActionMap  *action_map,
 {
   GtkApplicationWindow *window = GTK_APPLICATION_WINDOW (action_map);
 
+  if (!window->priv->actions)
+    return;
+
   g_action_map_remove_action (G_ACTION_MAP (window->priv->actions), action_name);
 }
 
@@ -555,7 +565,7 @@ gtk_application_window_real_get_preferred_width (GtkWidget *widget,
       gtk_widget_get_preferred_width (window->priv->menubar, &menubar_min_width, &menubar_nat_width);
 
       border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
-      _gtk_window_get_decoration_size (GTK_WINDOW (widget), &border);
+      _gtk_window_get_shadow_width (GTK_WINDOW (widget), &border);
 
       menubar_min_width += 2 * border_width + border.left + border.right;
       menubar_nat_width += 2 * border_width + border.left + border.right;
@@ -591,7 +601,7 @@ gtk_application_window_real_get_preferred_width_for_height (GtkWidget *widget,
       gtk_widget_get_preferred_width_for_height (window->priv->menubar, menubar_height, &menubar_min_width, &menubar_nat_width);
 
       border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
-      _gtk_window_get_decoration_size (GTK_WINDOW (widget), &border);
+      _gtk_window_get_shadow_width (GTK_WINDOW (widget), &border);
 
       menubar_min_width += 2 * border_width + border.left + border.right;
       menubar_nat_width += 2 * border_width + border.left + border.right;
@@ -639,10 +649,8 @@ static void
 gtk_application_window_real_realize (GtkWidget *widget)
 {
   GtkApplicationWindow *window = GTK_APPLICATION_WINDOW (widget);
-  GtkApplication *application;
   GtkSettings *settings;
 
-  application = gtk_window_get_application (GTK_WINDOW (window));
   settings = gtk_widget_get_settings (widget);
 
   g_signal_connect (settings, "notify::gtk-shell-shows-app-menu",
@@ -655,35 +663,6 @@ gtk_application_window_real_realize (GtkWidget *widget)
   gtk_application_window_update_shell_shows_app_menu (window, settings);
   gtk_application_window_update_shell_shows_menubar (window, settings);
   gtk_application_window_update_menubar (window);
-
-#ifdef GDK_WINDOWING_X11
-  {
-    GdkWindow *gdkwindow;
-
-    gdkwindow = gtk_widget_get_window (GTK_WIDGET (window));
-
-    if (GDK_IS_X11_WINDOW (gdkwindow) && window->priv->session)
-      {
-        gdk_x11_window_set_utf8_property (gdkwindow, "_GTK_APPLICATION_ID",
-                                          g_application_get_application_id (G_APPLICATION (application)));
-
-        gdk_x11_window_set_utf8_property (gdkwindow, "_GTK_UNIQUE_BUS_NAME",
-                                          g_dbus_connection_get_unique_name (window->priv->session));
-
-        gdk_x11_window_set_utf8_property (gdkwindow, "_GTK_APPLICATION_OBJECT_PATH",
-                                          g_application_get_dbus_object_path (G_APPLICATION (application)));
-
-        gdk_x11_window_set_utf8_property (gdkwindow, "_GTK_WINDOW_OBJECT_PATH",
-                                          window->priv->object_path);
-
-        gdk_x11_window_set_utf8_property (gdkwindow, "_GTK_APP_MENU_OBJECT_PATH",
-                                          gtk_application_get_app_menu_object_path (application));
-
-        gdk_x11_window_set_utf8_property (gdkwindow, "_GTK_MENUBAR_OBJECT_PATH",
-                                          gtk_application_get_menubar_object_path (application));
-      }
-  }
-#endif
 }
 
 static void
@@ -699,46 +678,10 @@ gtk_application_window_real_unrealize (GtkWidget *widget)
   GTK_WIDGET_CLASS (gtk_application_window_parent_class)->unrealize (widget);
 }
 
-gboolean
-gtk_application_window_publish (GtkApplicationWindow *window,
-                                GDBusConnection      *session,
-                                const gchar          *object_path,
-                                guint                 object_id)
+GActionGroup *
+gtk_application_window_get_action_group (GtkApplicationWindow *window)
 {
-  g_assert (window->priv->session == NULL);
-  g_assert (window->priv->export_id == 0);
-  g_assert (window->priv->object_path == NULL);
-  g_assert (window->priv->id == 0);
-
-  window->priv->export_id = g_dbus_connection_export_action_group (session, object_path,
-                                                                   G_ACTION_GROUP (window->priv->actions),
-                                                                   NULL);
-
-  if (window->priv->export_id == 0)
-    return FALSE;
-
-  window->priv->session = session;
-  window->priv->object_path = g_strdup (object_path);
-  window->priv->id = object_id;
-
-  return TRUE;
-}
-
-void
-gtk_application_window_unpublish (GtkApplicationWindow *window)
-{
-  g_assert (window->priv->session != NULL);
-  g_assert (window->priv->export_id != 0);
-  g_assert (window->priv->object_path != NULL);
-  g_assert (window->priv->id != 0);
-
-  g_dbus_connection_unexport_action_group (window->priv->session, window->priv->export_id);
-  window->priv->session = NULL;
-  window->priv->export_id = 0;
-  window->priv->id = 0;
-
-  g_free (window->priv->object_path);
-  window->priv->object_path = NULL;
+  return G_ACTION_GROUP (window->priv->actions);
 }
 
 static void
@@ -750,27 +693,6 @@ gtk_application_window_real_map (GtkWidget *widget)
   if (window->priv->menubar)
     gtk_widget_map (window->priv->menubar);
 
-#ifdef GDK_WINDOWING_WAYLAND
-  {
-    GdkWindow *gdkwindow;
-    GtkApplication *application;
-
-    application = gtk_window_get_application (GTK_WINDOW (window));
-    gdkwindow = gtk_widget_get_window (widget);
-
-    if (GDK_IS_WAYLAND_WINDOW (gdkwindow) && window->priv->session)
-      {
-	gdk_wayland_window_set_dbus_properties_libgtk_only (gdkwindow,
-							    g_application_get_application_id (G_APPLICATION (application)),
-							    gtk_application_get_app_menu_object_path (application),
-							    gtk_application_get_menubar_object_path (application),
-							    window->priv->object_path,
-							    g_application_get_dbus_object_path (G_APPLICATION (application)),
-							    g_dbus_connection_get_unique_name (window->priv->session));
-      }
-  }
-#endif
-
   GTK_WIDGET_CLASS (gtk_application_window_parent_class)->map (widget);
 }
 
@@ -840,10 +762,18 @@ gtk_application_window_dispose (GObject *object)
 
   g_clear_object (&window->priv->app_menu_section);
   g_clear_object (&window->priv->menubar_section);
-  g_clear_object (&window->priv->actions);
 
   G_OBJECT_CLASS (gtk_application_window_parent_class)
     ->dispose (object);
+
+  /* We do this below the chain-up above to give us a chance to be
+   * removed from the GtkApplication (which is done in the dispose
+   * handler of GtkWindow).
+   *
+   * That reduces our chances of being watched as a GActionGroup from a
+   * muxer constructed by GtkApplication.
+   */
+  g_clear_object (&window->priv->actions);
 }
 
 static void
@@ -994,3 +924,11 @@ gtk_application_window_get_id (GtkApplicationWindow *window)
 
   return window->priv->id;
 }
+
+void
+gtk_application_window_set_id (GtkApplicationWindow *window,
+                               guint                 id)
+{
+  g_return_if_fail (GTK_IS_APPLICATION_WINDOW (window));
+  window->priv->id = id;
+}
