@@ -136,6 +136,7 @@ static const GtkTargetEntry drop_types[] = { { "application/x-color", 0, 0 } };
 static void gtk_color_button_iface_init (GtkColorChooserInterface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (GtkColorButton, gtk_color_button, GTK_TYPE_BUTTON,
+                         G_ADD_PRIVATE (GtkColorButton)
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_COLOR_CHOOSER,
                                                 gtk_color_button_iface_init))
 
@@ -257,8 +258,6 @@ gtk_color_button_class_init (GtkColorButtonClass *klass)
                                                   NULL, NULL,
                                                   _gtk_marshal_VOID__VOID,
                                                   G_TYPE_NONE, 0);
-
-  g_type_class_add_private (gobject_class, sizeof (GtkColorButtonPrivate));
 }
 
 static gboolean
@@ -427,9 +426,7 @@ gtk_color_button_init (GtkColorButton *button)
   PangoRectangle rect;
 
   /* Create the widgets */
-  button->priv = G_TYPE_INSTANCE_GET_PRIVATE (button,
-                                              GTK_TYPE_COLOR_BUTTON,
-                                              GtkColorButtonPrivate);
+  button->priv = gtk_color_button_get_instance_private (button);
 
   button->priv->draw_area = gtk_drawing_area_new ();
   layout = gtk_widget_create_pango_layout (GTK_WIDGET (button), "Black");
@@ -570,6 +567,7 @@ dialog_response (GtkDialog *dialog,
 
       gtk_widget_queue_draw (button->priv->draw_area);
 
+      g_object_ref (button);
       g_signal_emit (button, color_button_signals[COLOR_SET], 0);
 
       g_object_freeze_notify (G_OBJECT (button));
@@ -577,6 +575,7 @@ dialog_response (GtkDialog *dialog,
       g_object_notify (G_OBJECT (button), "alpha");
       g_object_notify (G_OBJECT (button), "rgba");
       g_object_thaw_notify (G_OBJECT (button));
+      g_object_unref (button);
     }
 }
 
