@@ -404,9 +404,9 @@ gtk_info_bar_hide (GtkWidget *widget)
 {
   GtkInfoBarPrivate *priv = GTK_INFO_BAR (widget)->priv;
 
-  gtk_revealer_set_reveal_child (GTK_REVEALER (priv->revealer), FALSE);
   g_signal_connect_object (priv->revealer, "notify::child-revealed",
                            G_CALLBACK (child_revealed), widget, 0);
+  gtk_revealer_set_reveal_child (GTK_REVEALER (priv->revealer), FALSE);
 }
 
 static void
@@ -1184,7 +1184,7 @@ gtk_info_bar_set_message_type (GtkInfoBar     *info_bar,
         {
           const char *name = NULL;
 
-          atk_object_set_role (atk_obj, ATK_ROLE_ALERT);
+          atk_object_set_role (atk_obj, ATK_ROLE_INFO_BAR);
 
           switch (message_type)
             {
@@ -1213,9 +1213,7 @@ gtk_info_bar_set_message_type (GtkInfoBar     *info_bar,
             }
 
           if (name)
-            {
-              atk_object_set_name (atk_obj, name);
-            }
+            atk_object_set_name (atk_obj, name);
         }
 
       if (type_class[priv->message_type])
