@@ -67,10 +67,15 @@ typedef gint (*GtkKeySnoopFunc) (GtkWidget   *grab_widget,
 
 /* GTK+ version
  */
+GDK_AVAILABLE_IN_ALL
 guint gtk_get_major_version (void) G_GNUC_CONST;
+GDK_AVAILABLE_IN_ALL
 guint gtk_get_minor_version (void) G_GNUC_CONST;
+GDK_AVAILABLE_IN_ALL
 guint gtk_get_micro_version (void) G_GNUC_CONST;
+GDK_AVAILABLE_IN_ALL
 guint gtk_get_binary_age    (void) G_GNUC_CONST;
+GDK_AVAILABLE_IN_ALL
 guint gtk_get_interface_age (void) G_GNUC_CONST;
 
 #define gtk_major_version gtk_get_major_version ()
@@ -79,6 +84,7 @@ guint gtk_get_interface_age (void) G_GNUC_CONST;
 #define gtk_binary_age gtk_get_binary_age ()
 #define gtk_interface_age gtk_get_interface_age ()
 
+GDK_AVAILABLE_IN_ALL
 const gchar* gtk_check_version (guint   required_major,
                                 guint   required_minor,
                                 guint   required_micro);
@@ -87,15 +93,19 @@ const gchar* gtk_check_version (guint   required_major,
 /* Initialization, exit, mainloop and miscellaneous routines
  */
 
+GDK_AVAILABLE_IN_ALL
 gboolean gtk_parse_args           (int    *argc,
                                    char ***argv);
 
+GDK_AVAILABLE_IN_ALL
 void     gtk_init                 (int    *argc,
                                    char ***argv);
 
+GDK_AVAILABLE_IN_ALL
 gboolean gtk_init_check           (int    *argc,
                                    char ***argv);
 
+GDK_AVAILABLE_IN_ALL
 gboolean gtk_init_with_args       (gint                 *argc,
                                    gchar              ***argv,
                                    const gchar          *parameter_string,
@@ -103,6 +113,7 @@ gboolean gtk_init_with_args       (gint                 *argc,
                                    const gchar          *translation_domain,
                                    GError              **error);
 
+GDK_AVAILABLE_IN_ALL
 GOptionGroup *gtk_get_option_group (gboolean open_default_display);
 
 #ifdef G_OS_WIN32
@@ -110,11 +121,13 @@ GOptionGroup *gtk_get_option_group (gboolean open_default_display);
 /* Variants that are used to check for correct struct packing
  * when building GTK+-using code.
  */
+GDK_AVAILABLE_IN_ALL
 void     gtk_init_abi_check       (int    *argc,
                                    char ***argv,
                                    int     num_checks,
                                    size_t  sizeof_GtkWindow,
                                    size_t  sizeof_GtkBox);
+GDK_AVAILABLE_IN_ALL
 gboolean gtk_init_check_abi_check (int    *argc,
                                    char ***argv,
                                    int     num_checks,
@@ -126,27 +139,50 @@ gboolean gtk_init_check_abi_check (int    *argc,
 
 #endif
 
+GDK_AVAILABLE_IN_ALL
 void           gtk_disable_setlocale    (void);
+GDK_AVAILABLE_IN_ALL
 PangoLanguage *gtk_get_default_language (void);
+GDK_AVAILABLE_IN_3_12
+GtkTextDirection gtk_get_locale_direction (void);
+GDK_AVAILABLE_IN_ALL
 gboolean       gtk_events_pending       (void);
 
+#ifndef GTK_DISABLE_DEPRECATED
+void     gtk_exit                 (gint    error_code);
+gchar *        gtk_set_locale           (void);
+#endif /* GTK_DISABLE_DEPRECATED */
+
+GDK_AVAILABLE_IN_ALL
 void       gtk_main_do_event       (GdkEvent           *event);
+GDK_AVAILABLE_IN_ALL
 void       gtk_main                (void);
+GDK_AVAILABLE_IN_ALL
 guint      gtk_main_level          (void);
+GDK_AVAILABLE_IN_ALL
 void       gtk_main_quit           (void);
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_main_iteration      (void);
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_main_iteration_do   (gboolean            blocking);
 
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_true                (void) G_GNUC_CONST;
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_false               (void) G_GNUC_CONST;
 
+GDK_AVAILABLE_IN_ALL
 void       gtk_grab_add            (GtkWidget          *widget);
+GDK_AVAILABLE_IN_ALL
 GtkWidget* gtk_grab_get_current    (void);
+GDK_AVAILABLE_IN_ALL
 void       gtk_grab_remove         (GtkWidget          *widget);
 
+GDK_AVAILABLE_IN_ALL
 void       gtk_device_grab_add     (GtkWidget          *widget,
                                     GdkDevice          *device,
                                     gboolean            block_others);
+GDK_AVAILABLE_IN_ALL
 void       gtk_device_grab_remove  (GtkWidget          *widget,
                                     GdkDevice          *device);
 
@@ -156,13 +192,19 @@ guint      gtk_key_snooper_install (GtkKeySnoopFunc snooper,
 GDK_DEPRECATED_IN_3_4
 void       gtk_key_snooper_remove  (guint           snooper_handler_id);
 
+GDK_AVAILABLE_IN_ALL
 GdkEvent * gtk_get_current_event        (void);
+GDK_AVAILABLE_IN_ALL
 guint32    gtk_get_current_event_time   (void);
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_get_current_event_state  (GdkModifierType *state);
+GDK_AVAILABLE_IN_ALL
 GdkDevice *gtk_get_current_event_device (void);
 
+GDK_AVAILABLE_IN_ALL
 GtkWidget *gtk_get_event_widget         (GdkEvent        *event);
 
+GDK_AVAILABLE_IN_ALL
 void       gtk_propagate_event          (GtkWidget       *widget,
                                          GdkEvent        *event);
 
