@@ -258,12 +258,23 @@ struct _GtkStyleContextClass
 /**
  * GTK_STYLE_CLASS_MENU:
  *
- * A CSS class to match popup menus.
+ * A CSS class to match menus.
  *
  * This is used in #GtkMenu.
  */
 #define GTK_STYLE_CLASS_MENU "menu"
 
+/**
+ * GTK_STYLE_CLASS_CONTEXT_MENU:
+ *
+ * A CSS class to match context menus.
+ *
+ * This style class is useful when you want to prevent
+ * a context menu from inheriting e.g. font changes from
+ * the widget it is attached to.
+ */
+#define GTK_STYLE_CLASS_CONTEXT_MENU "context-menu"
+
 /**
  * GTK_STYLE_CLASS_MENUBAR:
  *
@@ -735,6 +746,24 @@ struct _GtkStyleContextClass
  */
 #define GTK_STYLE_CLASS_INSERTION_CURSOR "insertion-cursor"
 
+/**
+ * GTK_STYLE_CLASS_TITLEBAR:
+ *
+ * A CSS class used when rendering a titlebar in a toplevel
+ * window.
+ */
+#define GTK_STYLE_CLASS_TITLEBAR "titlebar"
+
+/**
+ * GTK_STYLE_CLASS_NEEDS_ATTENTION:
+ *
+ * A CSS class used when an element needs the user attention,
+ * for instance a button in a stack switcher corresponding to
+ * a hidden page that changed state.
+*
+ * Since: 3.12
+ */
+#define GTK_STYLE_CLASS_NEEDS_ATTENTION "needs-attention"
 
 /* Predefined set of widget regions */
 
@@ -905,6 +934,12 @@ void           gtk_style_context_set_frame_clock (GtkStyleContext *context,
 GDK_AVAILABLE_IN_3_8
 GdkFrameClock *gtk_style_context_get_frame_clock (GtkStyleContext *context);
 
+/**
+ * GTK_STYLE_CLASS_READ_ONLY:
+ *
+ * A CSS class used to indicate a read-only state.
+ */
+#define GTK_STYLE_CLASS_READ_ONLY "read-only"
 
 GDK_DEPRECATED_IN_3_8_FOR(gtk_style_context_set_state)
 void             gtk_style_context_set_direction (GtkStyleContext  *context,
@@ -975,7 +1010,7 @@ void gtk_style_context_get_margin           (GtkStyleContext *context,
                                              GtkStateFlags    state,
                                              GtkBorder       *margin);
 
-GDK_AVAILABLE_IN_ALL
+GDK_DEPRECATED_IN_3_12
 void gtk_style_context_invalidate           (GtkStyleContext *context);
 GDK_AVAILABLE_IN_ALL
 void gtk_style_context_reset_widgets        (GdkScreen       *screen);
