@@ -1478,7 +1478,7 @@ set_stroke_style (cairo_t        *cr,
           n = round ((1. / 3) * n);
 
           segments[0] = n ? (1. / 3) * length / n : 1;
-          segments[1] = 2 * segments[1];
+          segments[1] = 2 * segments[0];
         }
       cairo_set_dash (cr, segments, G_N_ELEMENTS (segments), 0);
 
@@ -1591,6 +1591,13 @@ render_border (cairo_t       *cr,
       if (hidden_side & (1 << i))
         continue;
 
+      /* NB: code below divides by this value */
+      /* a border smaller than this will not noticably modify
+       * pixels on screen, and since we don't compare with 0,
+       * we'll use this value */
+      if (border_width[i] < 1.0 / 1024)
+        continue;
+
       switch (border_style[i])
         {
         case GTK_BORDER_STYLE_NONE:
@@ -2058,10 +2065,9 @@ gtk_theming_engine_render_frame_gap (GtkThemingEngine *engine,
   GtkStateFlags state;
   gint border_width;
   GtkCssValue *corner[4];
-  gdouble x0, y0, x1, y1, xc, yc, wc, hc;
+  gdouble x0, y0, x1, y1, xc = 0.0, yc = 0.0, wc = 0.0, hc = 0.0;
   GtkBorder border;
 
-  xc = yc = wc = hc = 0;
   state = gtk_theming_engine_get_state (engine);
   junction = gtk_theming_engine_get_junction_sides (engine);
 
