@@ -53,8 +53,8 @@ typedef gboolean (* GtkTreeModelFilterVisibleFunc) (GtkTreeModel *model,
  * GtkTreeModelFilterModifyFunc:
  * @model: the #GtkTreeModelFilter
  * @iter: a #GtkTreeIter pointing to the row whose display values are determined
- * @value: A #GValue which is already initialized for with the correct type for
- *  the column @column.
+ * @value: (out caller-allocates): A #GValue which is already initialized for
+ *  with the correct type for the column @column.
  * @column: the column whose display value is determined
  * @data: (closure): user data given to gtk_tree_model_filter_set_modify_func()
  *
@@ -105,38 +105,50 @@ struct _GtkTreeModelFilterClass
 };
 
 /* base */
+GDK_AVAILABLE_IN_ALL
 GType         gtk_tree_model_filter_get_type                   (void) G_GNUC_CONST;
+GDK_AVAILABLE_IN_ALL
 GtkTreeModel *gtk_tree_model_filter_new                        (GtkTreeModel                 *child_model,
                                                                 GtkTreePath                  *root);
+GDK_AVAILABLE_IN_ALL
 void          gtk_tree_model_filter_set_visible_func           (GtkTreeModelFilter           *filter,
                                                                 GtkTreeModelFilterVisibleFunc func,
                                                                 gpointer                      data,
                                                                 GDestroyNotify                destroy);
+GDK_AVAILABLE_IN_ALL
 void          gtk_tree_model_filter_set_modify_func            (GtkTreeModelFilter           *filter,
                                                                 gint                          n_columns,
                                                                 GType                        *types,
                                                                 GtkTreeModelFilterModifyFunc  func,
                                                                 gpointer                      data,
                                                                 GDestroyNotify                destroy);
+GDK_AVAILABLE_IN_ALL
 void          gtk_tree_model_filter_set_visible_column         (GtkTreeModelFilter           *filter,
                                                                 gint                          column);
 
+GDK_AVAILABLE_IN_ALL
 GtkTreeModel *gtk_tree_model_filter_get_model                  (GtkTreeModelFilter           *filter);
 
 /* conversion */
+GDK_AVAILABLE_IN_ALL
 gboolean      gtk_tree_model_filter_convert_child_iter_to_iter (GtkTreeModelFilter           *filter,
                                                                 GtkTreeIter                  *filter_iter,
                                                                 GtkTreeIter                  *child_iter);
+GDK_AVAILABLE_IN_ALL
 void          gtk_tree_model_filter_convert_iter_to_child_iter (GtkTreeModelFilter           *filter,
                                                                 GtkTreeIter                  *child_iter,
                                                                 GtkTreeIter                  *filter_iter);
+GDK_AVAILABLE_IN_ALL
 GtkTreePath  *gtk_tree_model_filter_convert_child_path_to_path (GtkTreeModelFilter           *filter,
                                                                 GtkTreePath                  *child_path);
+GDK_AVAILABLE_IN_ALL
 GtkTreePath  *gtk_tree_model_filter_convert_path_to_child_path (GtkTreeModelFilter           *filter,
                                                                 GtkTreePath                  *filter_path);
 
 /* extras */
+GDK_AVAILABLE_IN_ALL
 void          gtk_tree_model_filter_refilter                   (GtkTreeModelFilter           *filter);
+GDK_AVAILABLE_IN_ALL
 void          gtk_tree_model_filter_clear_cache                (GtkTreeModelFilter           *filter);
 
 G_END_DECLS
