@@ -467,6 +467,9 @@ GDK_AVAILABLE_IN_3_10
 void     gtk_window_set_titlebar           (GtkWindow    *window,
                                             GtkWidget    *titlebar);
 
+GDK_AVAILABLE_IN_3_12
+gboolean gtk_window_is_maximized           (GtkWindow    *window);
+
 G_END_DECLS
 
 #endif /* __GTK_WINDOW_H__ */
