@@ -65,9 +65,6 @@ void            _gtk_window_set_allocation         (GtkWindow           *window,
                                                     const GtkAllocation *allocation,
                                                     GtkAllocation       *allocation_out);
 
-gboolean       _gtk_window_handle_button_press_for_widget (GtkWidget      *widget,
-                                                           GdkEventButton *event);
-
 typedef void (*GtkWindowKeysForeachFunc) (GtkWindow      *window,
                                           guint           keyval,
                                           GdkModifierType modifiers,
@@ -89,8 +86,10 @@ void            _gtk_window_notify_keys_changed (GtkWindow *window);
 
 gboolean        _gtk_window_titlebar_shows_app_menu (GtkWindow *window);
 
-void            _gtk_window_get_decoration_size (GtkWindow *window,
-                                                 GtkBorder *border);
+void            _gtk_window_get_shadow_width (GtkWindow *window,
+                                              GtkBorder *border);
+
+void            _gtk_window_toggle_maximized (GtkWindow *window);
 
 G_END_DECLS
 
